/*!
 * pixi-controller - v1.1.0
 *
 * pixi-controller is licensed under the MIT License.
 * http://www.opensource.org/licenses/mit-license
 */
var _pixi_factory = (function (exports) {
	'use strict';

	//const format = require("string-format");


	class EventElement {
		constructor(subEventName, isAsync, funct) {
			this.subEventName = subEventName;
			this.isAsync = isAsync;
			this.funct = funct;
		}
	}

	/**
	 * Singular event.
	 */
	class Event {
		constructor(parent) {
			this.partent = parent;
			this.elementsMap = new Map();
			this.elements = new Array();
		}

		getParent() {
			return this.parent;
		}

		condition(...args) {
			let result = true;

			// Run
			this.elements.some(element => {
				if (!element.funct(...args)) {
					result = false;
					return false;
				}
			});

			return result;
		}

		call(...args) {
			this.elements.some(element => {
				element.funct(...args);
			});
		}

		has(subEventName) {
			return this.elementsMap.has(subEventName);
		}

		remove(subEventName) {
			if (!this.has(subEventName)) return false;

			this.elementsMap.delete(subEventName);
			this.elements.splice(this._getIndex(subEventName), 1);

			return true;
		}

		delete(subEventName) {
			this.remove(subEventName);
		}

		clear() {
			this.elementsMap.clear();
			this.elements.length = 0;
		}

		replace(subEventName, funct) {
			if (!this.has(subEventName)) return false;

			this.elementsMap.get(subEventName).funct = funct;
			this.elements[this._getIndex(subEventName)].funct = funct;

			return true;
		}
		prepend(subEventName, funct, isAsyncOpt) {
			let obj = new EventElement(subEventName, isAsyncOpt, funct);
			this.elementsMap.set(subEventName, obj);
			this.elements.unshift(obj);
		}

		append(subEventName, funct, isAsyncOpt) {
			let obj = new EventElement(subEventName, isAsyncOpt, funct);
			this.elementsMap.set(subEventName, obj);
			this.elements.push(obj);
		}

		insertBefore(subEventNameSeek, subEventName, funct, isAsyncOpt) {
			//throw new Error(format("Error, Event: Insert seek key not found '{0}'.", subEventNameSeek));
			let index = this._getIndex(subEventNameSeek);

			let obj = new EventElement(subEventName, isAsyncOpt, funct);
			this.elementsMap.set(subEventName, obj);
			this.elements.splice(index, 0, obj);
		}

		insertAfter(subEventNameSeek, subEventName, funct, isAsyncOpt) {
			let index = this._getIndex(subEventNameSeek);

			let obj = new EventElement(subEventName, isAsyncOpt, funct);
			this.elementsMap.set(subEventName, obj);
			this.elements.splice(index + 1, 0, obj);
		}

		_getIndex(subEventNameSeek) {
			return this.elements.findIndex(element => element.subEventName == subEventNameSeek);
		}
	}

	/**
	 * Event handler.
	 */
	class Events {
		constructor() {
			this.elementsMap = new Map();
		}

		condition(eventName, ...args) {
			return new Promise((resolve, reject) => {
				let element = this.elementsMap.get(eventName);
				
				if (element)
					resolve(element.condition());

				resolve(true); // Do not block if no conditions
			});
		}
	  
	  emit(...args) {
	    this.call(...args);
	  }

		call(eventName, ...args) {
			return new Promise((resolve, reject) => {
				let element;
				
				element = this.elementsMap.get('pre_' + eventName);
				if (element)
					element.call(...args);

				element = this.elementsMap.get(eventName);
				if (element)
					element.call(...args);

				element = this.elementsMap.get('post_' + eventName);
				if (element)
					element.call(...args);

				resolve(); // No fancy async currently
			});

			//	throw new Error(format("Error, Events: Failed to call '{0}'.", eventName));
		}

		has(eventName, subEventName) {
			if (!this.elementsMap.has(eventName)) return false;

			if (!subEventName) return true;

			return this.elementsMap.get(eventName).has(subEventName);
		}

		remove(eventName, subEventName) {
			if (!this.has(eventName)) return false;

			if (!subEventName) {
				this.elementsMap.delete(eventName);
				return true;
			}

			return this.elementsMap.get(eventName).remove(subEventName);
		}

		delete(eventName, subEventName) {
			this.remove(eventName, subEventName);
		}

		clear(eventName) {
			if (!eventName) {
				this.elementsMap.clear();
				return;
			}

			this.elementsMap.get(eventName).clear();
		}

		prepend(eventName, subEventName, funct, isAsyncOpt) {
			this._attainEvent(eventName).prepend(subEventName, funct, isAsyncOpt);
		}
	  
	  on(...args) {
	    this.append(...args);
	  }

		append(eventName, subEventName, funct, isAsyncOpt) {
	    if (typeof subEventName === "function") {
	      let values = Object.values(arguments);
	      values.splice(1, 0, '_');
	      this.append(...values);
	      return;
	    }
	    
	    this._attainEvent(eventName).append(subEventName, funct, isAsyncOpt);
		}

		insertBefore(eventName, subEventName, subEventNameSeek, funct, isAsyncOpt) {
	    if (typeof subEventNameSeek === "function") {
	      let values = Object.values(arguments);
	      values.splice(1, 0, '_');
	      this.insertBefore(...values);
	      return;
	    }
	    
			this._attainEvent(eventName).insertBefore(subEventName, subEventNameSeek, funct, isAsyncOpt);
		}

		insertAfter(eventName, subEventName, subEventNameSeek, funct, isAsyncOpt) {
	    if (typeof subEventNameSeek === "function") {
	      let values = Object.values(arguments);
	      values.splice(1, 0, '_');
	      this.insertAfter(...values);
	      return;
	    }
	    
			this._attainEvent(eventName).insertAfter(subEventName, subEventNameSeek, funct, isAsyncOpt);
		}

		_attainEvent(eventName) {
			if (!this.elementsMap.has(eventName)) this.elementsMap.set(eventName, new Event(this));

			return this.elementsMap.get(eventName);
		}
	}

	var Events_1 = Events;

	var E__dev_libs_pixiController_node_modules_nomEvents = Events_1;

	var Mouse = /** @class */ (function () {
	    function Mouse(canvas) {
	        var _this = this;
	        this.posLocalY = 0;
	        this.posLocalX = 0;
	        this.posGlobalX = 0;
	        this.posGlobalY = 0;
	        this._prevent = {
	            value: false,
	            code: [],
	        };
	        this.mouseMoveListener = function (event) {
	            var buttonCode = event.button;
	            _this.preventMiddle(buttonCode, event);
	            if (_this.posLocalX != event.clientX || _this.posLocalY != event.clientY) {
	                _this.events.call('move', event.clientX, _this.posLocalY);
	                _this.events.call('moveLocal', event.clientX, _this.posLocalY);
	                _this.posLocalX = event.clientX;
	                _this.posLocalY = event.clientY;
	            }
	            if (_this.posGlobalX != event.screenX || _this.posGlobalY != event.screenY) {
	                _this.events.call('moveGlobal', event.screenX, event.screenY);
	                _this.posGlobalX = event.screenX;
	                _this.posGlobalY = event.screenY;
	            }
	        };
	        this.mouseDownListener = function (event) {
	            var buttonCode = event.button;
	            _this.preventMiddle(buttonCode, event);
	            if (!_this.buttonStates.get(buttonCode)) {
	                event.posLocalX = _this.getPosLocalX();
	                event.posLocalY = _this.getPosLocalY();
	                _this.buttonStates.set(buttonCode, event);
	                _this.events.call('pressed', buttonCode, event, _this.getPosLocalX(), _this.getPosLocalY());
	                _this.events.call('pressed_' + buttonCode, buttonCode, event, _this.getPosLocalX(), _this.getPosLocalY());
	            }
	        };
	        this.mouseUpListener = function (event) {
	            var buttonCode = event.button;
	            _this.preventMiddle(buttonCode, event);
	            event = _this.buttonStates.get(buttonCode);
	            if (event) {
	                event.wasReleased = true;
	                _this.events.call('released', buttonCode, event, _this.getPosLocalX(), _this.getPosLocalY(), event.posLocalX, event.posLocalY, _this.getPosLocalX() - event.posLocalX, _this.getPosLocalY() - event.posLocalY);
	                _this.events.call('released_' + buttonCode, buttonCode, event, _this.getPosLocalX(), _this.getPosLocalY(), event.posLocalX, event.posLocalY, _this.getPosLocalX() - event.posLocalX, _this.getPosLocalY() - event.posLocalY);
	            }
	        };
	        this.mouseMenuContext = function (event) {
	            _this.preventMiddle(event.button, event);
	        };
	        this.buttonStates = new Map();
	        this.events = new E__dev_libs_pixiController_node_modules_nomEvents();
	        this.canvasElement = canvas;
	        if (!this.canvasElement)
	            throw new Error('pixi-controller: error for search <canvas /> tag.');
	        this.canvasElement.addEventListener('mousemove', this.mouseMoveListener);
	        this.canvasElement.addEventListener('mousedown', this.mouseDownListener);
	        this.canvasElement.addEventListener('mouseup', this.mouseUpListener);
	        this.canvasElement.addEventListener('contextmenu', this.mouseMenuContext);
	    }
	    Mouse.prototype.getMousePos = function (event) {
	        var rect = this.canvasElement.getBoundingClientRect();
	        return {
	            x: event.clientX - rect.left,
	            y: event.clientY - rect.top,
	        };
	    };
	    Mouse.prototype.getPosGlobalX = function () {
	        return this.posGlobalX;
	    };
	    Mouse.prototype.getPosGlobalY = function () {
	        return this.posGlobalY;
	    };
	    Mouse.prototype.getPosLocalX = function () {
	        return this.posLocalX;
	    };
	    Mouse.prototype.getPosLocalY = function () {
	        return this.posLocalY;
	    };
	    Mouse.prototype.getPosX = function () {
	        return this.getPosLocalX();
	    };
	    Mouse.prototype.getPosY = function () {
	        return this.getPosLocalY();
	    };
	    Mouse.prototype.clear = function () {
	        this.buttonStates.clear();
	    };
	    Mouse.prototype.reset = function (_canvas) {
	        _canvas ? (this.canvasElement = _canvas) : (this.canvasElement = document.getElementsByTagName('canvas')[0]);
	        this.clear();
	        this.events = new E__dev_libs_pixiController_node_modules_nomEvents();
	        this.canvasElement.removeEventListener('mousemove', this.mouseMoveListener);
	        this.canvasElement.removeEventListener('mousedown', this.mouseDownListener);
	        this.canvasElement.removeEventListener('mouseup', this.mouseUpListener);
	        this.canvasElement.removeEventListener('contextmenu', this.mouseMenuContext);
	        this.canvasElement.addEventListener('mousemove', this.mouseMoveListener);
	        this.canvasElement.addEventListener('mousedown', this.mouseDownListener);
	        this.canvasElement.addEventListener('mouseup', this.mouseUpListener);
	        this.canvasElement.addEventListener('contextmenu', this.mouseMenuContext);
	    };
	    Mouse.prototype.update = function () {
	        var _this = this;
	        this.buttonStates.forEach(function (value, buttonCode) {
	            var event = _this.buttonStates.get(buttonCode);
	            event.alreadyPressed = true;
	            if (event.wasReleased)
	                _this.buttonStates.delete(buttonCode);
	            _this.events.call('down', buttonCode, event, _this.getPosLocalX(), _this.getPosLocalY(), event.posLocalX, event.posLocalY, _this.getPosLocalX() - event.posLocalX, _this.getPosLocalY() - event.posLocalY);
	            _this.events.call('down_' + buttonCode, buttonCode, event, _this.getPosLocalX(), _this.getPosLocalY(), event.posLocalX, event.posLocalY, _this.getPosLocalX() - event.posLocalX, _this.getPosLocalY() - event.posLocalY);
	        });
	    };
	    Mouse.prototype.isButtonDown = function () {
	        var args = [];
	        for (var _i = 0; _i < arguments.length; _i++) {
	            args[_i] = arguments[_i];
	        }
	        var result = false;
	        for (var _a = 0, args_1 = args; _a < args_1.length; _a++) {
	            var buttonCode = args_1[_a];
	            var key = this.buttonStates.get(buttonCode);
	            if (key && !key.wasReleased)
	                result = true;
	        }
	        return result;
	    };
	    Mouse.prototype.isButtonUp = function () {
	        var args = [];
	        for (var _i = 0; _i < arguments.length; _i++) {
	            args[_i] = arguments[_i];
	        }
	        return !this.isButtonDown(args);
	    };
	    Mouse.prototype.isButtonPressed = function () {
	        var args = [];
	        for (var _i = 0; _i < arguments.length; _i++) {
	            args[_i] = arguments[_i];
	        }
	        var result = false;
	        if (args.length == 0)
	            return false;
	        for (var _a = 0, args_2 = args; _a < args_2.length; _a++) {
	            var buttonCode = args_2[_a];
	            var event_1 = this.buttonStates.get(buttonCode);
	            if (event_1 && !event_1.wasReleased && !event_1.alreadyPressed)
	                result = true;
	        }
	        return result;
	    };
	    Mouse.prototype.isButtonReleased = function () {
	        var args = [];
	        for (var _i = 0; _i < arguments.length; _i++) {
	            args[_i] = arguments[_i];
	        }
	        var result = false;
	        if (args.length == 0)
	            return false;
	        for (var _a = 0, args_3 = args; _a < args_3.length; _a++) {
	            var buttonCode = args_3[_a];
	            var event_2 = this.buttonStates.get(buttonCode);
	            if (event_2 && event_2.wasReleased)
	                result = true;
	        }
	        return result;
	    };
	    Mouse.prototype.prevent = function () {
	        var _this = this;
	        var args = [];
	        for (var _i = 0; _i < arguments.length; _i++) {
	            args[_i] = arguments[_i];
	        }
	        this._prevent.value = true;
	        args.forEach(function (arg) {
	            _this._prevent.code.push(arg);
	        });
	    };
	    Mouse.prototype.preventRemove = function () {
	        this._prevent.value = false;
	        this._prevent.code = [];
	    };
	    Mouse.prototype.preventMiddle = function (button, event) {
	        var _this = this;
	        this._prevent.code.forEach(function (cd) {
	            if (_this._prevent.value && button === cd) {
	                event.preventDefault();
	                event.stopPropagation();
	            }
	        });
	    };
	    return Mouse;
	}());

	var Keyboard = /** @class */ (function () {
	    function Keyboard() {
	        var _this = this;
	        this.keyDownListener = function (event) {
	            if (!_this.keyStates.get(event.code)) {
	                _this.keyStates.set(event.code, event);
	                _this.events.call('pressed', event.code, event);
	                _this.events.call('pressed_' + event.code, event.code, event);
	            }
	        };
	        this.keyUpListener = function (event) {
	            event = _this.keyStates.get(event.code);
	            if (event) {
	                event.wasReleased = true;
	                _this.events.call('released', event.code, event);
	                _this.events.call('released_' + event.code, event.code, event);
	            }
	        };
	        this.keyStates = new Map();
	        this.events = new E__dev_libs_pixiController_node_modules_nomEvents();
	        this.set();
	    }
	    Keyboard.prototype.clear = function () {
	        this.keyStates.clear();
	    };
	    Keyboard.prototype.update = function () {
	        var _this = this;
	        this.keyStates.forEach(function (value, keyCode) {
	            var event = _this.keyStates.get(keyCode);
	            event.alreadyPressed = true;
	            if (event.wasReleased) {
	                _this.keyStates.delete(keyCode);
	            }
	            _this.events.call('down', keyCode, event);
	            _this.events.call('down_' + keyCode, keyCode, event);
	        });
	    };
	    Keyboard.prototype.isKeyDown = function () {
	        var args = [];
	        for (var _i = 0; _i < arguments.length; _i++) {
	            args[_i] = arguments[_i];
	        }
	        var result = false;
	        for (var _a = 0, args_1 = args; _a < args_1.length; _a++) {
	            var keyCode = args_1[_a];
	            var event_1 = this.keyStates.get(keyCode);
	            if (event_1 && !event_1.wasReleased)
	                result = true;
	        }
	        return result;
	    };
	    Keyboard.prototype.isKeyUp = function () {
	        var args = [];
	        for (var _i = 0; _i < arguments.length; _i++) {
	            args[_i] = arguments[_i];
	        }
	        return !this.isKeyDown(args);
	    };
	    Keyboard.prototype.isKeyPressed = function () {
	        var args = [];
	        for (var _i = 0; _i < arguments.length; _i++) {
	            args[_i] = arguments[_i];
	        }
	        var result = false;
	        if (args.length == 0)
	            return false;
	        for (var _a = 0, args_2 = args; _a < args_2.length; _a++) {
	            var keyCode = args_2[_a];
	            var event_2 = this.keyStates.get(keyCode);
	            if (event_2 && !event_2.wasReleased && !event_2.alreadyPressed)
	                result = true;
	        }
	        return result;
	    };
	    Keyboard.prototype.isKeyReleased = function () {
	        var args = [];
	        for (var _i = 0; _i < arguments.length; _i++) {
	            args[_i] = arguments[_i];
	        }
	        var result = false;
	        if (args.length == 0)
	            return false;
	        for (var _a = 0, args_3 = args; _a < args_3.length; _a++) {
	            var keyCode = args_3[_a];
	            var event_3 = this.keyStates.get(keyCode);
	            if (event_3 && event_3.wasReleased)
	                result = true;
	        }
	        return result;
	    };
	    Keyboard.prototype.set = function () {
	        window.addEventListener('keydown', this.keyDownListener, false);
	        window.addEventListener('keyup', this.keyUpListener, false);
	    };
	    Keyboard.prototype.reset = function () {
	        this.clear();
	    };
	    return Keyboard;
	}());

	var PIXIController = /** @class */ (function () {
	    function PIXIController() {
	        var _this = this;
	        setTimeout(function () {
	            _this.Canvas = document.getElementsByTagName('canvas')[0];
	            _this.Mouse = new Mouse(_this.Canvas);
	            _this.Keyboard = new Keyboard();
	        }, 0);
	    }
	    PIXIController.prototype.update = function () {
	        this.Mouse.update();
	        this.Keyboard.update();
	    };
	    PIXIController.prototype.reset = function () {
	        this.Mouse.reset();
	        this.Keyboard.reset();
	    };
	    PIXIController.prototype.setContext = function (_canvas) {
	        this.Canvas = _canvas;
	        if (!this.Mouse || !this.Keyboard)
	            throw new Error('Controller cannot be instantiable before the loader.');
	        this.Mouse.reset(_canvas);
	    };
	    PIXIController.prototype.getContext = function () {
	        return this.Canvas;
	    };
	    PIXIController.prototype.prevent = function () {
	        this.Canvas.removeEventListener('contextmenu', function (event) {
	            event.preventDefault();
	            event.stopPropagation();
	        });
	        this.Canvas.addEventListener('contextmenu', function (event) {
	            event.preventDefault();
	            event.stopPropagation();
	        });
	    };
	    return PIXIController;
	}());

	var BUTTON = {
	    LEFT: 0,
	    MIDDLE: 1,
	    RIGHT: 2,
	    FOURTH: 3,
	    FIFTH: 4,
	};
	var PLAYER = {
	    UP: ['ArrowUp', 'KeyW'],
	    DOWN: ['ArrowDown', 'KeyS'],
	    LEFT: ['ArrowLeft', 'KeyA'],
	    RIGHT: ['ArrowRight', 'KeyD'],
	    INVENTORY: ['KeyI'],
	    BAR: ['Digit1', 'Digit2', 'Digit3', 'Digit4', 'Digit5', 'Digit6', 'Digit7', 'Digit8', 'Digit9'],
	};

	var index = new PIXIController();

	exports.BUTTON = BUTTON;
	exports.PLAYER = PLAYER;
	exports.default = index;

	Object.defineProperty(exports, '__esModule', { value: true });

	return exports;

}({}));
//# sourceMappingURL=index.iife.js.map
